//file: invariants.hpp  Version 1.01  Jan 8, 2004
#ifndef INVARIANTS_HPP
#define INVARIANTS_HPP

// Copyright Daniel Watkins 2004. Use, modification and distribution
//    is subject to the Boost Software License, Version 1.0.
//         <http://www.boost.org/LICENSE_1_0.txt>
// See "invariants.txt" for preliminary documentation.

#ifdef __DMC__
# define USE_INT_3_INSTEAD_OF_ASSERT
#endif

#ifdef USE_INT_3_INSTEAD_OF_ASSERT
# ifdef _DEBUG
#  ifdef assert
#   undef assert
#  endif
#  define assert(x) do{if(!((x)))asm int 3;}while(0)
# else
#  define assert(x) do{(void)(0);}while(0)
# endif
#else
# include <cassert>
#endif

namespace boost { namespace DBC {

template< typename Derived >
struct invariants
{
#ifdef _DEBUG
    typedef invariants< Derived > invars_t;
    struct trigobj
    {
        trigobj( invars_t const * pinv )
        : pinv_( pinv )
        {
            assert( pinv_ );
            if( pinv_ )
                pinv_->verf_invars();
        }
        ~trigobj()
        {
            assert( pinv_ );
            if( pinv_ )
                pinv_->verf_invars();
            pinv_ = 0;
        }
     private:
        invars_t const * pinv_;
        trigobj(); //no default; needs the pointer.
        trigobj( trigobj const & );
        void operator=( trigobj const & );
    };
    invariants(invars_t const &){}
    invars_t & operator=(invars_t const &){ return *this; }
    void verf_invars() const
    {
        verify_invariants(this);
    }
 protected:
    invariants(){}
    ~invariants()
    {
    }
 private:
    virtual void verify_invariants(invars_t const *) const = 0;
#endif
};
                }               }

#define INVARIANTS_BASE  private invariants

#ifdef _DEBUG

# define INVARIANTS() friend struct invariants< type >; \
void verify_invariants(invariants< type > const *) const
# define _INVARIANTS_CHECKED_ invariants< type >::trigobj TRGBJ(this);

#else //Release mode:

# define INVARIANTS() template<int ZZZ> void zzz() const
# define _INVARIANTS_CHECKED_

#endif

#endif

